//+------------------------------------------------------------------+
//|                                                     3_Shadow.mqh |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "220Volt"


#include <Shadow_lib\\1_Cursor.mqh>
#include <Shadow_lib\\1_Direction.mqh>
#include <Shadow_lib\\1_Point_gen.mqh>
#include <Shadow_lib\\2_Level_cross.mqh>


 //  . k  : WindowBarsPerChart() * k * 2
#define shadow.SHADOW_WIDTH    0.03


//+------------------------------------------------------------------+
//| private data                                                     |
//+------------------------------------------------------------------+
int shadow.timeframe;
int shadow.direction;
int shadow.seg[][2];
double shadow.seg_p[][2];

//+------------------------------------------------------------------+
//| private                                                          |
//+------------------------------------------------------------------+
string shadow.name(string base,
                   int index)
{ 
   return(  StringConcatenate( base, DoubleToStr(index, 0) )  ); 
}

bool shadow.timeframe_selection(datetime time,
                                bool permission)
{
   if( ! permission )
   {
      shadow.timeframe = Period();
      return(true);
   }
   int tf[] = {1, 5, 15, 30, 60, 240, 1440, 10080, 43200};
   for(int i = 0;  i < ArraySize(tf);  i ++)
   {
      datetime cur_time = iTime(NULL, tf[i], iBars(NULL, tf[i]) - 1);
      if(cur_time == 0)
         return(false);
      if(cur_time <= time)
      {
         shadow.timeframe = tf[i];
         return(true);
      }
   }
   return(false);
}

bool shadow.determine_direction(datetime &shadow_time[][2])
{
   if( ArrayRange(shadow_time, 0) == 1 )
   {
      switch( cursor.sector_position() )
      {
      case 0:
      case 1:
      case 2:
         shadow.direction = direction.DOWN;
         return(true);
        
      case 6:
      case 7:
      case 8:
         shadow.direction = direction.UP;
         return(true);
         
      default: return(false);
      }
   }
    
   int first_middle = iBarShift(NULL, shadow.timeframe, shadow_time[0][0]) - 
                     (
                       iBarShift(NULL, shadow.timeframe, shadow_time[0][0]) - 
                       iBarShift(NULL, shadow.timeframe, shadow_time[0][1])
                     ) / 2;
   
   int second_middle = iBarShift(NULL, shadow.timeframe, shadow_time[1][0]) - 
                       (
                         iBarShift(NULL, shadow.timeframe, shadow_time[1][0]) - 
                         iBarShift(NULL, shadow.timeframe, shadow_time[1][1])
                       ) / 2;

   double first_mprice = iClose(NULL, shadow.timeframe, first_middle);
   double second_mprice = iClose(NULL, shadow.timeframe, second_middle);
   if(first_mprice == 0  ||  second_mprice == 0)
      return(false);
   
   if(first_mprice > second_mprice)
      shadow.direction = direction.DOWN;
   else
      shadow.direction = direction.UP;
   
   return(true);
}

bool shadow.extrem_search(int start_time,
                          int end_time,
                          int direction,
                          int seg_index)
{
   int start_bar = iBarShift(NULL, shadow.timeframe, start_time);
   int end_bar = iBarShift(NULL, shadow.timeframe, end_time);
   
   if(direction == direction.UP)
      point_gen.init(start_bar,
                     point_gen.LOW,
                     end_bar,
                     point_gen.HIGH,
                     shadow.timeframe);
   else
      point_gen.init(start_bar,
                     point_gen.HIGH,
                     end_bar,
                     point_gen.LOW,
                     shadow.timeframe);
      
   shadow.seg[seg_index][0] = start_bar;
   shadow.seg[seg_index][1] = start_bar;
   shadow.seg_p[seg_index][0] = point_gen.get();
   shadow.seg_p[seg_index][1] = shadow.seg_p[seg_index][0];
   if(shadow.seg_p[seg_index][0] == 0)
      return(false);
   
   while( point_gen.is_full() )
   {
      double cur_price = point_gen.get();
      if(cur_price == 0)
         return(false);
         
      switch( cross_checkD(shadow.seg_p[seg_index][0], cur_price, direction) )
      {
      case level_cross.first_cross_second:
         shadow.seg[seg_index][0] = point_gen.last_bar_index();
         shadow.seg[seg_index][1] = shadow.seg[seg_index][0];
         shadow.seg_p[seg_index][0] = cur_price;
         shadow.seg_p[seg_index][1] = cur_price;
         continue;
         
      default: break;
      }
      
      switch( cross_checkD(shadow.seg_p[seg_index][1], cur_price, direction) )
      {
      case level_cross.second_cross_first:
         shadow.seg[seg_index][1] = point_gen.last_bar_index();
         shadow.seg_p[seg_index][1] = cur_price;
         
      default: break;
      }
   }
   
   return(true);
}

//+------------------------------------------------------------------+
//| public                                                           |
//+------------------------------------------------------------------+
int shadow.count(string name)
{
   int ret = 0;
   while(true)
   {
      if( ObjectFind( shadow.name(name, ret) ) == -1 )
         break;
      ret ++;
   }
   return(ret);
}

void shadow.create(string name,
                   color clr = C'0,0,81')
{
   name = shadow.name( name, shadow.count(name) );

   int cursor_pos = cursor.bar_position();
   if(cursor_pos == -1)
      return;
 
   int width = WindowBarsPerChart() * (shadow.SHADOW_WIDTH * 1.0);
      
   datetime left_time = Time[cursor_pos] - width * Period() * 60;
   datetime right_time = Time[cursor_pos] + width * Period() * 60;
   
   if(Bars - 1 > cursor_pos)
      if( Time[cursor_pos + 1] < left_time )
         left_time = Time[cursor_pos + 1];
         
   if(cursor_pos > 0)
      if( Time[cursor_pos - 1] > right_time)
         right_time = Time[cursor_pos - 1];
         
   ObjectCreate(name, OBJ_RECTANGLE, 0, 
                left_time,
                WindowPriceMax(),
                right_time,
                WindowPriceMin() );
   ObjectSet(name, OBJPROP_COLOR, clr);
   ObjectSet(name, OBJPROP_BACK, true);
   
   return;
}

void shadow.delete(string name)
{
   int count = shadow.count(name);
   for(count --;  count >= 0;  count --)
      ObjectDelete( shadow.name(name, count) );
   return;
}

bool shadow.attach(string name,
                   bool select_timeframe = true)
{
   datetime shadow_time[][2];
   //begin//
      int count = shadow.count(name);
      if(count < 1)
         return(false);
   
      if( ArrayResize(shadow_time, count) == -1 )
         return(false);
  
      for(count --;  count >= 0;  count --)
      {     
         shadow_time[count][0] = ObjectGet(shadow.name(name, count), OBJPROP_TIME1);
         shadow_time[count][1] = ObjectGet(shadow.name(name, count), OBJPROP_TIME2);
      }
      ArraySort(shadow_time);
   
      for(int shadow_time_sz = ArrayRange(shadow_time, 0) - 1;  shadow_time_sz >= 1;  shadow_time_sz --)
      {
         if(shadow_time[shadow_time_sz - 1][1] >= shadow_time[shadow_time_sz][1])
            return(false);
         if(shadow_time[shadow_time_sz - 1][0] >= shadow_time[shadow_time_sz][0])
            return(false);
      }
   //end//
   if( ! shadow.timeframe_selection(shadow_time[0][0], select_timeframe) )
      return(false);
   if( ! shadow.determine_direction(shadow_time) )
      return(false);
   
   if(ArrayRange(shadow_time, 0) == 1)
   {
      if( ArrayResize(shadow.seg, 1) == -1 )
         return(false);
      if( ArrayResize(shadow.seg_p, 1) == -1 )
         return(false);
      if( ! shadow.extrem_search(shadow_time[0][0],
                                 shadow_time[0][1],
                                 shadow.direction, 0) )
         return(false);
         
   }
   else
   {
      if( ArrayResize(shadow.seg, ArrayRange(shadow_time, 0) - 1) == -1 )
         return(false);
      if( ArrayResize(shadow.seg_p, ArrayRange(shadow_time, 0) - 1) == -1 )
         return(false);
         
      int cur_direction = shadow.direction;   
      for(int position = 0;  position < ArrayRange(shadow_time, 0) - 1;  position ++)
      {
         if( ! shadow.extrem_search(shadow_time[position][0],
                                    shadow_time[position + 1][1],
                                    cur_direction, 
                                    position) )
            return(false);
            
         if(position > 0)
            if( shadow.seg[position - 1][1] != shadow.seg[position][0] )
            {
               shadow.seg[position - 1][1] = shadow.seg[position][0];
               shadow.seg_p[position - 1][1] = shadow.seg_p[position][0];
            }
            
         cur_direction = direction.reverse(cur_direction);
      }
   }
   
   return(true);
}

bool shadow.cut(double depth,
                int segment_index = 0)
{
   int cur_direction;
   if( (segment_index & 1) == true )
      cur_direction = direction.reverse(shadow.direction); 
   else
      cur_direction = shadow.direction;
   
   if(cur_direction == direction.UP)
      point_gen.init(shadow.seg[segment_index][0],
                     point_gen.LOW,
                     shadow.seg[segment_index][1],
                     point_gen.HIGH,
                     shadow.timeframe);
   else
      point_gen.init(shadow.seg[segment_index][0],
                     point_gen.HIGH,
                     shadow.seg[segment_index][1],
                     point_gen.LOW,
                     shadow.timeframe);
      
   shadow.seg[segment_index][1] = shadow.seg[segment_index][0];
   shadow.seg_p[segment_index][1] = shadow.seg_p[segment_index][0];
   
   while( point_gen.is_full() )
   {
      double cur_price = point_gen.get();
      if(cur_price == 0)
         return(false);
        
      if( cross_checkD(shadow.seg_p[segment_index][1], 
                       cur_price, cur_direction) == level_cross.second_cross_first )
      {
         shadow.seg[segment_index][1] = point_gen.last_bar_index();
         shadow.seg_p[segment_index][1] = cur_price;
         continue;
      } 
      
      double level;
      if(cur_direction == direction.UP)
         level = shadow.seg_p[segment_index][1] - 
                 (
                   shadow.seg_p[segment_index][1] - 
                   shadow.seg_p[segment_index][0]
                 ) * depth;
      else
         level = shadow.seg_p[segment_index][1] + 
                 (
                   shadow.seg_p[segment_index][0] - 
                   shadow.seg_p[segment_index][1]
                 ) * depth;
                 
      switch( cross_checkD(level, cur_price, cur_direction) )
      {
      case level_cross.equal:
      case level_cross.first_cross_second:   
         shadow.seg[segment_index][0] = point_gen.last_bar_index();
         shadow.seg[segment_index][1] = shadow.seg[segment_index][0];
         shadow.seg_p[segment_index][0] = cur_price;
         shadow.seg_p[segment_index][1] = cur_price;
      break;
         
      default: break;
      }
  }
  
  return(true);   
}

int shadow.get_start_bar(int segment_index = 0)        { return(shadow.seg[segment_index][0]); }

int shadow.get_end_bar(int segment_index = 0)          { return(shadow.seg[segment_index][1]); }

double shadow.get_start_price(int segment_index = 0)   { return(shadow.seg_p[segment_index][0]); }

double shadow.get_end_price(int segment_index = 0)     { return(shadow.seg_p[segment_index][1]); }

datetime shadow.get_start_time(int segment_index = 0)  { return( iTime(NULL, shadow.timeframe, shadow.seg[segment_index][0]) ); }

datetime shadow.get_end_time(int segment_index = 0)    { return( iTime(NULL, shadow.timeframe, shadow.seg[segment_index][1]) ); }

int shadow.get_timeframe()                             { return(shadow.timeframe); }